@@ -749,7 +749,7 @@ class Dlinterface:
 ################################################
 
 
-    def query(self, query=None, qtype='sql', fmt='csv', out=None, async_=False, profile='default', verbose=True, **kw):
+    def query(self, query=None, qtype='sql', fmt='csv', out=None, async_=False, drop=False, profile='default', verbose=True, **kw):
         '''
         Send a query to a remote query service.
 
@@ -797,6 +797,10 @@ class Dlinterface:
             was promoted to a keyword in Python 3.7. Users of Python versions
             prior to 3.7 can continue to use the ``async`` keyword.
 
+        drop : bool
+            If ``True``, then if the query is saving to mydb where the same table name
+            already exists, it will overwrite the old mydb table.
+
         Returns
         -------
         result : str
@@ -929,7 +933,7 @@ class Dlinterface:
 
         try:
             res = queryClient.query (token, adql=adql, sql=sql,
-                                     fmt=qcfmt, out=out, async_=async_)
+                                     fmt=qcfmt, out=out, async_=async_, drop=drop)
         except Exception as e:
             if not async_ and str(e) is not None:
                 err = str(e)
